{<skip/>}
{********************************
* (C) 2010 onerobot.org         *
* Unified PASM Driver for TBOT  *
*********************************
}

{
Comments
---------------------------------------------------

ConfigureMotorPWM
  The motor driver can be assigned a PWM frequency and a minimum PWM period. 
  Since it takes a certain amount of power to the motor to cause any movement at all,
  the minumum PWM correction allows the control input to vary continously over the
  range of PWM values that actually cause motion.

SetMotorBrake
  When motor power = 0, the h-bridge will normally be set to coast (no power) but
  there is also the option to apply an electronic brake (short motor windings together).
  Changing the value of break/coast will not automatically change the current speed 
  and the brake only takes effect when that motor is off (power = 0)
  
}
CON
  _clkmode      = xtal1 + pll16x
  _clkfreq      = 96_000_000

  ' Constants for sensor types
  LineSensors      = 1
  ProximitySensors = 2
  EncoderSensors   = 4
  AllSensors       = 7
  
  ' Constants for sensor modes
  NormalMode       = %000
  RawMode          = %001
  CalibrateMode    = %010
  EncoderMode      = %100

  ' Constants for sensor data types
  MinValue         = 0
  MaxValue         = 1
  Param0           = 2
  Param1           = 3
  Rate             = 4
  Value            = 5
    
  MAXPOWER         = 1000
  MIDPOWER         = 500
  TRACK_SIZE       = 1
  DRIVE_CAL        = 1
  LEFT_CAL         = 1
  RIGHT_CAL        = 1
  
VAR
  long debugParam[2]           ' Used for debugging overall driver status
  
  long motorAddr, sensorAddr  ' pointers to motor and sensor control data in HUB RAM
  
  long motorCmd       ' Command and Status register
  long motorMode
  long motorPwm[2]    ' left and right wheel power

  long motorMaxPwm
  long motorConfig
  long motorMinPwm
    
  long sensorMode
  long sensorRate

  long lineData[5*6]  ' 6 longs for each: min cal, max cal, param0, param1, rate, value
  long proxData[2*6]
  long encoderData[2*6]

  long sensorCount    ' counter incremented after each sensor update cycle    
  long lastLinePos
  
  
  long line[5]
  long dbg[2]
        
  long outP
  long inP
  long mtr[2]
  long temp
  long last_p, temp_i,straight
  
OBJ
  vp   : "Terminal"
  i2c  : "t_i2c"
  qs   : "QuickSample" 
  
pub Main | frame[400], i,j ''Test harness
  vp.register(qs.sampleINA(@frame,1))
  
  vp.config(string("var:io(bits=[0m0,1m1,2m2,3m3,23s0,21s1,20s2,19s3,17s4,8debug]),outp,inp,mtrA,mtrB,temp,turn(min=-1000,max=1000),speed(min=-1000,max=1000),line_p,line_i,line_d,factor_p(min=1000,max=100000),factor_i(min=1000,max=50000),factor_d(min=1000,max=10000),max_speed(min=0,max=1000)"))
  vp.config(string("start:lsa::terminal:1"))
  vp.config(string("lsa:view=io,timescale=2ms"))
  vp.config(string("dso:view=[temp,line_p,line_i,line_d,mtrA,mtrB]"))
  vp.config(string("edit:factor_p(mode=scroll),factor_i(mode=scroll),factor_d(mode=scroll),max_speed(mode=scroll)"))
    
  vp.share(@outP,@temp)                             '*** Configure ViewPort to share variables 
    
  i2c.Start

  ' Initialize PID parameters
  ' 5000, 10000, 500, 200
  ' 500, 10000, 200, 400
  ' 200, 8000, 30, 500
  Start
  i2c.SetIRLED(0) 
pub dopid(error,factor_p,factor_i,factor_d)|temp_p,temp_d,line_p,line_i,line_d
    ' Calculate raw PID values
    temp_p := error
    temp_i += temp_p
    temp_d := temp_p - last_p
    last_p := temp_p
    
    ' Calculate scaled PID values
    line_p := temp_p * factor_p
    line_i := temp_i * factor_i
    line_d := temp_d * factor_d
    return (line_p+line_i+line_d)/1000
pub constrain(v,vmin,vmax)
    return (v <# vmax) #> vmin        
pub FollowLine | i, j,power_diff,max_speed ''Sample program
  max_speed := 500
  last_p := GetLinePos
  temp_i := 0
    
  repeat
    repeat j from 0 to 4
      line[j] := GetNormalLine(j)
    
    temp := GetLinePos
    i2c.SetHSL((temp + 2000) * 255 / 4000, 255, 100)
    power_diff := constrain(dopid(temp,200,5000,30),-max_speed,max_speed)
{   if power_diff < 0
      SetMotorPower(max_speed + power_diff, max_speed)
    else
      SetMotorPower(max_speed, max_speed - power_diff)
}     
    SetMotorPower(max_speed+power_diff, max_speed - power_diff)         
'    i := i2c.CheckButton(i)
 '   if i2c.ButtonPress(i, %01)
 '      quit
         
    waitcnt (clkfreq / 100 + cnt)
    
  SetMotorPower(0,0)
pub pausemsec(msec) 'pauses for msec
  waitcnt(cnt+clkfreq/1000*||msec)  
pub DoCal(delay) ''Calibrate all sensors
  i2c.SetHSL(0, 255, 50)
  
  SetCalibration(AllSensors, -1, 0)
  SetSensorMode(AllSensors, CalibrateMode)
  
  PauseMSec(delay)
  
  i2c.SetHSL(0, 255, 50)
  
  SetMotorPower(-150, 150)
  PauseMSec(800)
  
  SetMotorPower(0,0)
  PauseMSec(100)
  
  i2c.SetHSL(100, 255, 50)
  
  SetMotorPower(150,-150)

  PauseMSec(1500)
  
  SetMotorPower(0,0)
  PauseMSec(100)
  
  i2c.SetHSL(200, 255, 50)
  
  SetMotorPower(-150, 150)
  PauseMSec(800)
  
  i2c.SetHSL(255, 255, 50)
  
  SetMotorPower(0,0)
  SetSensorMode(AllSensors, RawMode)
  
pub Start|m0,m1
  m0:=2
  m1:=1
  motorConfig := m0 << 24 | (m0+1) << 16 | m1 << 8 | (m1-1) '2,1->2310
  motorAddr := @motorCmd
  sensorAddr := @sensorMode

  SetSensorRefresh(300)         
  SetCalibration(AllSensors, -1, 0)
  SetSensorMode(AllSensors, RawMode)

  ConfigureMotorPwm(20000, 300)

  motorCmd := 0
  motorMode := %0001_0001
  SetMotorPower(0,0)
  i2c.setpower(1,1,0)
  return cognew(@init, @debugParam[0])

pri ConfigureMotorPwm(freq, minPower) 
  ' sets up motor PWM frequency and minimum PWM period
  motorMaxPwm := clkfreq / freq
  motorMinPwm := motorMaxPwm * minPower / MAXPOWER
  
pri ConfigureMotorPins(leftA, leftB, rightA, rightB)
  motorConfig :=  leftB << 24 | leftA << 16 | rightB << 8 | rightA

{ 2 Wheeled Robot Kinematic Equations
    v = (Vl + Vr) / 2
    w = (Vr - Vl) / L
    R = L/2 * (Vl+Vr) / (Vr - Vl)
    
    L : Distance Between Wheels
    v : linear velocity
    w : angular velocity
    Vl : speed of left wheel
    Vr : speed of right wheel
    R : instantaneous turning radius

    Calculating left and right wheel velocities given linear/angular
    Vr := w * L/2 + v
    Vl := 2v - Vr
    
    Calculating left and right wheel velocities given linear and radius
    R := L/2 * (2v) / w*L
    Vr := v / R * L/2 + v
    Vl := 2v - Vr
}
  
pub DriveTurn(v, w) | vr, vl 'drive turn with velocity and w
  vr := w * TRACK_SIZE + v
  vl := 2 * v - vr
 
  'vr *= DRIVE_CAL
  'vl *= DRIVE_CAL
  'vr /= RIGHT_CAL
  'vl /= LEFT_CAL
  
  SetMotorPower(vl, vr)
  
pub DriveRadius(v, R) | vr, vl 'drive turn with radius
  vr := v / R * TRACK_SIZE + v
  vl := 2 * v - vr
    
  'vr *= DRIVE_CAL
  'vl *= DRIVE_CAL
  'vr /= RIGHT_CAL
  'vl /= LEFT_CAL
  
  SetMotorPower(vl, vr)
pub SetMotorPower(left, right) 'set power
    mtr[0] := left
    mtr[1] := right
    
    repeat until (motorCmd & 1) == 0
  
    'motorMode &= !%1111_1111 'unlock brakes
    motorMode &= !%0111_0111 'unlock brakes
    if left < 0
        motorMode |= %0011_0000
    elseif left > 0
        motorMode |= %0001_0000
    
    if right < 0
        motorMode |= %0000_0011
    elseif right > 0
        motorMode |= %0000_0001
        
    left := (||left) <# MAXPOWER 
    right := (||right) <# MAXPOWER 
    
    if left == MAXPOWER
      motorMode |= %0100_0000
    if right == MAXPOWER
      motorMode |= %0000_0100
      
    ' Calculate the amount of time pwm signal should be high
    left := ((motorMaxPwm - motorMinPwm) * left) / MAXPOWER
    left += motorMinPwm

    right := ((motorMaxPwm - motorMinPwm) * right) / MAXPOWER
    right += motorMinPwm

    ' Motor driver expects amount of time pwm signal should be low as
    ' an offset from $8000 (time until high bit will be set by Counter)
    motorPwm[0] := $8000_0000 - (motorMaxPwm - left)
    motorPwm[1] := $8000_0000 - (motorMaxPwm - right)

    ' Calculate the amount of time pwm signal should be high
    
    ' Motor driver expects amount of time pwm signal should be low as
    ' an offset from $8000 (time until high bit will be set by Counter)
  
    ' Set command update flag
    motorCmd |= 1
    
pub SetMotorBrake(left, right) 'set brake
  repeat until (motorCmd & 1) == 0

  if left
    motorMode &= !%0111_0000
    motorMode |= %1000_0000
  else
    motorMode &= !%1000_0000
    
  if right
    motorMode &= !%0000_0111
    motorMode |= %0000_1000
  else
    motorMode &= !%0000_1000

  motorCmd |= 1
        
pub SetSensorRefresh(freq)  'set sensor refresh rate
  sensorRate := clkfreq / freq
  
pub SetSensorMode(sensor, md) | i, t  'set sensor mode

  if (sensor & LineSensors)
    t := 0
    repeat 5
      t <<= 2
      t |= md
    sensorMode := (sensorMode & %1111_1111_0000000000) | t
    
  if (sensor & ProximitySensors)
    t := md
    t <<= 2
    t |= md
    t <<= 10
    sensorMode := (sensorMode & %1111_0000_1111111111) | t
    
  
  if (sensor & EncoderSensors)
    t := md
    t <<= 2
    t |= md
    t <<= 14
    sensorMode := (sensorMode & %0000_1111_1111111111) | t
    
    
pri SetCalibration(sensor, minval, maxval) | i  'set calibration
  if (sensor & LineSensors)
    repeat i from 0 to 5
      lineData[i*6+0] := minval
      lineData[i*6+1] := maxval
  
  if (sensor & EncoderSensors)
    repeat i from 0 to 1
      encoderData[i*6+0] := minval
      encoderData[i*6+1] := maxval
  
  if (sensor & ProximitySensors)
    repeat i from 0 to 1
      proxData[i*6+0] := minval
      proxData[i*6+1] := maxval

pub GetLine(index, param)        'get a line sensor  
  return lineData[index*6+param]/330/4  '0..1000
  
pub GetLinePos | nsum, dsum, v, weight, i 'get calculated line position
  weight := 0
  repeat i from 0 to 4
    v := GetNormalLine(i)
    nsum += v * weight
    dsum += v
    weight += 1000
  
  if dsum > 200
    lastLinePos := (nsum / dsum) - 2000
  
  return lastLinePos
  
pub GetNormalLine(index) | i, minval, maxval, val 'get normalized line sensor
  i := index * 6
  minval := lineData[i+MinValue]
  maxval := lineData[i+MaxValue]
  
  val := (lineData[i+Value] <# maxval) #> minval
  val := ((val - minval) * 1000) / (maxval - minval)
  
  return val

pub GetProximity(index, param)
  return proxData[index*6+param]/320/5

pub GetEncoder(index, param)
  return encoderData[index*6+param]
     
    
DAT
org 0

Init
              mov   p0, par                     ' Store pointers to HUB RAM data structures
              add   p0, #8
              rdlong motor_root, p0
              add   p0, #4
              rdlong sensor_root, p0

              call #InitSensors
              call #InitDrive
                
              or  dira, debug
            
              
:mainloop             
              call  #DriveMotor          
              call  #SampleIR                          
             
              mov   t0, motor_window            ' track the maximum motor_window value for debugging
              call  #TimeIt              
              
              jmp   #:mainloop

InitSensors
              mov   p0, sensor_root             ' Read initial value of sensor refresh rate
              add   p0, #4
              rdlong sensor_wait, p0

              mov   sensor_start, cnt
              mov   sensor_stop, #0
              mov   sensor_curr, #SampleReset
              mov   sensor_next, #SampleReset
InitSensors_ret ret

InitDrive
              mov   p0, motor_root
              add   p0, #4*4
              rdlong motor_period, p0
              add   p0, #4
              rdlong motor_config, p0
                            
              call  #ConfigureDrive
                   
              mov   motor_start, cnt
              mov   motor_state, #DriveCheckCmd
                    
              mov   frqa, #1
              mov   frqb, #1
              or  dira, motor_mask0
              or  dira, motor_mask1
InitDrive_ret  ret

' Uses motor_mode and motor_config to setup pin masks and counter modes
ConfigureDrive
              ' Setup Left Motor 
              mov  t0, motor_config                       ' Get pin assignments from configuration
              mov  t1, motor_config
              shr  t0, #16
              shr  t1, #16
              test motor_mode, #%0010_0000 wz             ' Set t0 to high pin and t1 to low pin
        if_z  shr  t0, #8                                
        if_nz shr  t1, #8
              and  t0, #$FF
              and  t1, #$FF
              
              mov  motor_pinA0, #1                        ' Setup pin masks for high and low pin
              mov  motor_pinB0, #1
              shl  motor_pinA0, t0
              shl  motor_pinB0, t1
                            
              mov  motor_mask0, motor_pinA0               ' Setup pin mask for both pins
              or   motor_mask0, motor_pinB0
              
              mov  motor_ctrM0, pwm_mode                  ' Setup pwm counter mode for high pin
              add  motor_ctrM0, t0
                        
              ' Setup Right Motor 
              mov  t0, motor_config                       ' Get pin assignments from configuration
              mov  t1, motor_config
              test motor_mode, #%0000_0010 wz             ' Set t0 to high pin and t1 to low pin
        if_z  shr  t0, #8
        if_nz shr  t1, #8                                
              and  t0, #$FF
              and  t1, #$FF
              
              mov  motor_pinA1, #1                        ' Setup pin masks for high and low pin
              mov  motor_pinB1, #1
              shl  motor_pinA1, t0
              shl  motor_pinB1, t1
                            
              mov  motor_mask1, motor_pinA1               ' Setup pin mask for both pins
              or   motor_mask1, motor_pinB1
              
              mov  motor_ctrM1, pwm_mode                  ' Setup pwm counter mode for high pin
              add  motor_ctrM1, t0
              
ConfigureDrive_ret ret

DriveMotor 
              mov   t0, cnt
              sub   t0, motor_start
              add   t0, motor_window            ' safety margin for max time between checks
              cmp   t0, motor_period wc, wz
        if_b  jmp   motor_state
             
              
:check_window mov   t0, cnt                     ' determine if actual has exceeded cycle start
              sub   t0, motor_start
              cmp   t0, motor_period wc, wz
        if_ae jmp   #:fix_window                ' if we exceeded the next cycle start time then fix window 
        
:wait_start   
              mov   t1, motor_period
              sub   t1, t0
              add   t1, #8                      ' a little buffer to make sure we don't ever wrap the counter
              add   t1, cnt
              waitcnt t1, #0
              jmp   #:start_cycle 
              
:fix_window
              sub   t0, motor_period            ' update the window so next time we don't exceed cycle start 
              add   motor_window, t0            
  
:start_cycle              
              
              
              
              mov   motor_start, cnt
              
              mov   phsa, motor_pwm0            ' start timing the low pulse                          
              mov   phsb, motor_pwm1
              
              mov   ctra, timer_mode            ' release pins by putting counters in timer mode
              mov   ctrb, timer_mode
          
        test  motor_mode, #%0100_0000 wz  ' unless we are at 100% power set pwm signal low
        if_z  andn  outa, motor_mask0                              
              test  motor_mode, #%0000_0100 wz
        if_z  andn  outa, motor_mask1
        
            
:setup_left        
              test  motor_mode, #%0001_0000 wz  ' check for motor off or running
        if_nz jmp   #:left_running
              test  motor_mode, #%1000_0000 wz  ' check for motor stop or brake
        if_z  jmp   #:setup_right
              or    outa, motor_mask0           ' turn on the brake
              jmp   #:setup_right
:left_running
              andn  outa, motor_pinB0           ' turn low pin off
              test  motor_mode, #%0100_0000 wz  ' check if we are running 100%
         if_nz or    outa, motor_pinA0
         if_z  mov   ctra, motor_ctrM0
              
:setup_right
              test  motor_mode, #%0000_0001 wz  ' check for motor off or running
        if_nz jmp   #:right_running
              test  motor_mode, #%0000_1000 wz  ' check for motor coast or brake
        if_z  jmp   #DriveExit
              or    outa, motor_mask1           ' turn on the brake
              jmp   #DriveExit
:right_running
              andn  outa, motor_pinB1           ' turn low pin off
              
              test  motor_mode, #%0000_0100 wz  ' check if we are running 100%
        if_nz or    outa, motor_pinA1
        if_z  mov   ctrb, motor_ctrM1

              jmp   #DriveExit
                 
'131 - 260                      
DriveCheckCmd    
            
               
              mov   p0, motor_root               ' check if a drive command has been given
              rdlong t0, p0
              test  t0, #1 wz
        if_z  jmp #DriveExit

              add   p0, #4
              rdlong t1, p0                     ' read new motor mode bits
              add   p0, #4
              rdlong motor_pwm0, p0             ' read new left pwm value
              add   p0, #4
              rdlong motor_pwm1, p0             ' read new right pwm value
              
              andn  t0, #1                      ' update command with ACK status bit
              wrlong t0, motor_root
              
              mov  t0, motor_mode               
              mov  motor_mode, t1
              
              xor  t0, motor_mode               ' only need to reconfigure of direction changed
              and  t0, #%0010_0010 wz
        if_nz call #ConfigureDrive              
                 
              
DriveExit         
DriveMotor_ret ret
                                             
SampleIR
              mov   t0, cnt                    ' Check if we've exceeded the sensor_stop
              sub   t0, sensor_start
              cmp   t0, sensor_stop wc,wz
        
        if_b  jmp   sensor_curr                 ' Jump table for current state
              jmp   sensor_next                 ' Jump for next state after sensor_stop

SampleReset   mov   sensor_curr, #SampleExit    ' Wait for sensor_stop after reset code
              mov   sensor_next, #SampleBegin   ' Setup next state
              or    outa, pinmask
              or    dira, pinmask

              mov   sensor_start, cnt
              mov   sensor_stop, sensor_reset

              
              jmp   #SampleExit

SampleBegin   mov   sensor_curr, #SampleWait    ' Wait for sensor_stop after reset code
              mov   sensor_next, #SampleDone    ' Setup next state
              
              andn  dira, pinmask
              mov   sensor_pins, pinmask
              
              mov   sensor_start, cnt
              mov   sensor_stop, sensor_wait
              jmp   #SampleExit

SampleWait    mov   t1, ina                     ' Check if any pins have changed and record
              mov   t0, cnt                     ' the current time
              call  #CheckPins
              jmp   #SampleExit

SampleDone    mov   t1, #0                      ' Set max value for anything that hasn't reset yet
              mov   t0, cnt
              call  #CheckPins                  
              mov   sensor_next, #SampleUpdate
                                        
              jmp   #SampleExit
              
SampleUpdate  mov   sensor_ptr, sensor_root
              
              rdlong  sensor_mode, sensor_ptr
              add     sensor_ptr, #4
              rdlong  sensor_wait, sensor_ptr
              add     sensor_ptr, #4

              movs  SampleWrite, #timestop0
              movd  SampleSave,  #lastvalue0
              
              mov   sensor_index, #9             ' Save results to hub ram
              
              mov   sensor_next, #SampleWrite
                     
                                          
SampleWrite   mov   t0, 0-0 wz
              sub   t0, sensor_start
             
              test  sensor_mode, #%01 wz        ' Raw mode go straight to writing value
        if_nz add   sensor_ptr, #4*5                               
        if_nz jmp   #SampleSave
              test  sensor_mode, #%10 wz
        if_nz jmp   #:update_cal
              
:use_cal      rdlong  t3, sensor_ptr            ' t3 = minvalue
              add     sensor_ptr, #4
              rdlong  t4, sensor_ptr            ' t4 = maxvalue
              add     sensor_ptr, #4*4
              
              min     t0, t3                    ' clip output to calibration limits
              max     t0, t4
              
              sub     t0, t3
              'TODO: scale the output t0 := t0 * MAXOUTPUT / (maxvalue-minvalue)
              jmp     #SampleSave
              
:update_cal  
              xor   outa, debug
              
              rdlong  t3, sensor_ptr            ' values that have been seen are saved
              cmp     t0, t3 wc,wz
        if_b  wrlong  t0, sensor_ptr
              add     sensor_ptr, #4
              rdlong  t4, sensor_ptr
              cmp     t0, t4 wc,wz
        if_a  wrlong  t0, sensor_ptr
              add     sensor_ptr, #4*4
              
SampleSave    mov    0-0, t0
              wrlong t0, sensor_ptr
              add   sensor_ptr, #4
              
              add   SampleSave, dest_incr   
              add   SampleWrite, #1
                      
              shr   sensor_mode, #2 wr
              
              djnz  sensor_index, #SampleExit

              add sensor_count, #1                 ' update the sample counter
              wrlong sensor_count, sensor_ptr
              
              mov sensor_next, #SampleReset
                           
SampleExit
SampleIR_ret  ret

' Check all valid pins for a change since last check
' t1 := current pin state to check against (usually INA)
' t0 := current time to store if a change is detected (usually CNT)

CheckPins
              xor   t1,   sensor_pins         
              and   t1,   pinMask wz 
        if_z  jmp   #:exit
              andn  sensor_pins, t1           ' Save changes for next check
              
              movs  :check, #pinmask0
              movd  :store, #timestop0
                            
              mov   t2, #9                  ' Check all pins for changes
:check        test  t1,  0-0 wz
:store  if_nz mov   0-0, t0
              add   :check, #1
              add   :store, dest_incr
              djnz  t2, #:check
:exit
CheckPins_ret ret


' Simple Min/Max Timing Routing
' t0 : time delta to track maximum and minimum values
TimeIt
              mov   p0, par
              cmp   t0, debug_min wc,wz
        if_b  mov   debug_min, t0
        if_b  wrlong debug_min, p0
        
              add   p0, #4
              cmp   t0, debug_max wc,wz
        if_a  mov   debug_max, t0
        if_a  wrlong debug_max, p0

TimeIt_ret    ret

pwm_mode      long    %00100 << 26
timer_mode    long    %11111 << 26

dest_incr     long    %000000001_000000000


pinmask       long    %00000110_11111110_00000000_00000000
pinmask0      long    |<23,|<21,|<20,|<19,|<17,|<22,|<18,|<25,|<26
timestop0     long    1,2,3,4,5,6,7,8,9
lastvalue0    long    0,0,0,0,0,0,0,0,0


debug         long    |< 8

' Pointers to data structures in Hub Ram
motor_root    long    0
sensor_root   long    0                         

' Variable used by MotorDriver
motor_next    long    0
motor_mode    long    %0000_0000                ' defines how the motor should be controlled
motor_pwm0    long    $8000_0000 - 7200         ' period to keep left motor PWM signal off                                 
motor_pwm1    long    $8000_0000 - 4800         ' period to keep right motor PWM signal off
motor_period  long    9600                      ' total motor PWM period
motor_config  long    $00_01_02_03              ' configures how pins are connected to motors

motor_mask0   long    0                         ' pinmask for left motor off (or brake)
motor_pinA0   long    0                         ' high pin mask for left motor
motor_pinB0   long    0                         ' low pin mask for left motor
motor_ctrM0   long    0                         ' CTR mode value for pwm control of left motor

motor_mask1   long    0                         ' pinmask for right motor off (or brake)
motor_pinA1   long    0                         ' high pin mask for right motor
motor_pinB1   long    0                         ' low pin mask for right motor
motor_ctrM1   long    0                         ' CTR mode value for pwm control of right motor

motor_state   long    0                         ' motor state to execute while waiting for PWM complete
motor_start   long    0                         ' tracks start of current PWM cycle
motor_window  long    0                         ' compensation for longest time between state checks
              
' Variables used by SampleWrite state
sensor_pins   long    0                         ' keeps the last known state of the sensor input pins
sensor_start  long    0                         ' holds the start time of the current sample state
sensor_stop   long    0                         ' holds the time to wait before going to next state

sensor_reset  long    96 * 100                  ' amount of time the sensors capacitors are charged
sensor_wait   long    96_000_000 / 200          ' maximum time waited before starting a new sample

sensor_next   long    0                         ' state to run when timeout has been exceeded
sensor_curr   long    0                         ' state to run until timeout is reached
sensor_index  long    0                         ' index of current sensor being handled (write)
sensor_mode   long    0                         ' current run mode for each sensor
sensor_ptr    long    0                         ' pointer to hub ram for storing sample results
sensor_count  long    0

debug_start   long    0
debug_max     long    0
debug_min     long    -1

t0            res     1         ' Temporary variables
t1            res     1
t2            res     1
t3            res     1
t4            res     1
p0            res     1


FIT 372  '248 (50%), 372 (75%), 496 (100%)