#include <Arduino.h>
#include "VPmove.h"
#include "wiring_private.h"
//motor can be set -1000..1000
#define _maxpulses 5
#define _maxservos 2
static int _xx,_yy;
char _ldir=1,_rdir=1;
char goL,goR,stopL,stopR;
unsigned long lastL,lastR;
int pl,pr;
volatile int16_t pulsesL,pulsesR,speedL,speedR;


volatile int16_t l_dist,r_dist,l_remain,r_remain;
/*
go straight by stopping power to leading side-> left stops if right held.
keep target speed by controlling power-> torque varies as needed
manage distance by tracking pulses
*/
void pulseL(){
	unsigned long a=micros();
	speedL=a-lastL;
	lastL=a;
	pulsesL+=_ldir;	
	if(l_remain<r_remain) 
	    OCR0B =stopL;	  //before update was ahead, so now 1+ pulse ahead, so stop
	else 
	    OCR0B=goL;  //keep running
	if(l_remain) 
		l_remain-=r_dist; //remain is l_dist*r_dist, so after l_dist pulses should be at 0.	 
	else
		goL=stopL;  //make sure motor doesn't turn on after reach goal
	if(l_remain<=r_remain)
	    OCR0A=goR;  //after update is ahead, so power other one
}
void pulseR(){
	pulsesR+=_rdir;
	int a=micros();
	speedR=a-lastR;
	lastR=a;
	if(r_remain<l_remain) 
	    OCR0A =stopR;	  //before update was ahead, so now 1+ pulse ahead, so stop
	else 
	    OCR0A=goR;  //keep running
	if(r_remain) 
		r_remain-=l_dist; //remain is l_dist*r_dist, so after l_dist pulses should be at 0.	 
	else
		goR=stopR;  //make sure motor doesn't turn on after reach goal
	if(r_remain<=l_remain)
	    OCR0B=goL;  //after update is ahead, so power other one
}
/*at start of move set direction for each motor, then both distances are positive
d_remain is product of distances that's used to see how much left
*/
void _VPMove::set1(int al,int ar,int msec) { 
   int n,orig;   
   l_dist=abs(al);
   r_dist=abs(ar);
   if(l_dist==0) l_dist=1;
   if(r_dist==0) r_dist=1;
   orig=l_dist*r_dist;
   r_remain=orig;
   l_remain=orig;
   
   
   set(al>0?motor_cruise:-motor_cruise,ar>0?motor_cruise:-motor_cruise,false);   
  /* TODO: manage velocity
use cur_vel-target_vel to change nominal_pwm:
nominal_pwm+=(cur_vel<target_vel)*min(cur_vel*16,256)
target_vel=15 'start
target_vel=max(target_vel,min(end_vel,(max_dist-ramp_count)/8)) 'ramp down
'at least end_vel, at 80 ticks away vel=10.  40=5
target_vel=max(target_vel,min(ramp_count/2,cur_vel+1)) 'ramp up
'at beginning limited to distance away, at 10 ticks away vel=10
*/
/* todo: ramp servos
 each frame has position of all servos and msec till next frame
  loop interpolates between frames
  ensure time is accurate, pass ref to current time, updated
  interpolate so time is correct
  pointer to start frame gives all info
  movedata: frame,time....
  ramp(now,pin,num,start)
	time=start[num] //time in ms stored at end
	while 1{
		progress=(getms-now)/time
		if progress>1000 
			now+=time
			return
		progressi=1000-progress
		for n=0 to num-1
			pos[pin]=(progressi*start[n])+(start[n+num]*start[n+num])
		servoupd=true
	}
*/
   int total,last,stalled,ninv,nn;
   nn=100;
   stalled=0;   
   for(n=0;n<msec;n++) {  
	 total=l_remain+r_remain;
	 if(total==0) break;
	 switch (n) {
     case 0 ... 100: {
	   uint8_t oldSREG = SREG;
	   cli();
	   ninv=nn-n;
	   goL=(n*maxL+ninv*startL)/nn;goR=(n*maxR+ninv*startR)/nn;
	   //if(l_remain==orig || l_remain) 
	    OCR0A =goR;	  		
	   //if(r_remain==orig)
	    OCR0B=goL; 	   	   
	   SREG = oldSREG;	   
	   break;	   
	 }
	 default:
	   if(total==last) {
	 	stalled+=1;
		if(stalled>50) break; //pause after 100msec no motion
	   }	
	  else {
		last=total;
		stalled=0;
	  }
	 }
	 pauset(1);
	}	
	set(0,0,true);	
   return;
}
_VPMove::_VPMove(int a,int b,int _movec,int _turnc){	 
	  motor_cruise=500;
	  movec=_movec;
	  turnc=_turnc;	  
	  mmovec=movec;
	  mturnc=turnc;
	  gainl=256;
      gainr=256;
      ngainl=256;
      ngainr=256;
      zerol=1525;
      zeror=1525;	 	  
#if defined _MOVEHBRIDGE2
	ICR1 = 40000;   // frequency is every 20ms
	// Configure timer 1 for Fast PWM mode via ICR1, with 8x prescaling
	_xx = (1 << WGM11)|2 <<  6|2<<4;  // enable PWM on port B1,B2 in non-inverted compare mode 2
	_yy = (1 << WGM13) | (1<<WGM12) | (1 << CS11);	
	attachInterrupt(0, pulseL, CHANGE); //d2 for encoders
	attachInterrupt(1, pulseR, CHANGE); //d3
#define _MOVEHBRIDGE
      pinl=a;pinr=b;pinld=a+2;pinrd=b+2;	  
#else
	  pinl=a;pinr=b;pinld=a+2;pinrd=b+2;	  	  
#endif	
	pinMode(pinl, OUTPUT);
	pinMode(pinr, OUTPUT);  
}	
void _VPMove::init() {	
	uint8_t oldSREG = SREG;
	cli();
	TCCR0A |= _BV(COM0B1) | _BV(COM0A1);
	TCCR1A=_xx; //restore TCCR1A
	TCCR1B=_yy;	
	SREG = oldSREG;		
}
int _VPMove::speed(int side) {
	if(side==0)
		return speedL;
	return speedR;
}	
int _VPMove::pulses(int side) {
	if(side==0)
		return pulsesL;
	return pulsesR;
}	
void _VPMove::griptime(int shoulder,int hand,int ms){		
	int n,ss,hs,se,he;
	ss=(OCR1A-2000)/20;
	hs=(OCR1B-2000)/20;	
	se=shoulder>=0?shoulder-ss:0;
	he=hand>=0?hand-hs:0;	
	ms/=20;
	for(n=0;n<ms;n++) {
	  grip(se*n/ms+ss,he*n/ms+hs);
	  pauset(20);
	}
	grip(shoulder,hand);
}
void _VPMove::grip(int shoulder,int hand){		
    pinMode(9,  shoulder>=0?OUTPUT:INPUT); //servos
	OCR1A = shoulder*20+2000; // 2000/ms
	
	pinMode(10,  hand>=0?OUTPUT:INPUT); //servos
	OCR1B = hand*20+2000; // 
	
}
void _VPMove::calibrate(int _m,int _tr){
	mmovec=movec*_m/100;
	mturnc=turnc*_tr/100;
}
void _VPMove::set_cruise(int _x){
	motor_cruise=_x;
}
void _VPMove::correct(int _zl,int _zr,int _gl,int _gr,int _ngl,int _ngr){
}

void _VPMove::mm(int _mm,int _turn) {    
  //move and turn at cruise speed for specified time
  if (_mm<0) {
    _turn+=180; //reverse direction if msec is negative	
	_mm=-_mm;
  }
  _mm=_mm*28/10;
  _turn=(_turn+720) % 360; //'restrict to 0..360
  int msec=9999;
  
  switch (_turn) {
    case 270 ... 359:
	  set1(_mm*(_turn-315)/45,_mm,msec);
	  break;
    case 180 ... 269:
      set1(_mm*(_turn-225)/45,-_mm,msec);
	  break;
    case 91 ... 179:
      set1(-_mm,_mm*(135-_turn)/45,msec);
	  break;
    case 0 ... 90:
      set1(_mm,_mm*(45-_turn)/45,msec);
  }  
}
void _VPMove::deg(int deg,int _dir){
//turn at positive and negative cruise speed for specified time
  if (deg<0){
    deg=-deg;
	_dir=-_dir;
	}
  deg=deg*26/10;//245/90
  int msec=9999;
  if (_dir>0)
    set1(deg,-deg,msec);
  else
    set1(-deg,deg,msec);  
}
void _setpin(uint8_t pin,uint8_t mode){
	uint8_t bit = digitalPinToBitMask(pin);
	uint8_t port = digitalPinToPort(pin);
	volatile uint8_t *out,*reg;
	
	out = portOutputRegister(port);
	reg = portModeRegister(port);
	*reg |= bit; //set output		
	switch(mode){
	case HIGH:
		*out |= bit; //set high		
		break;
	case LOW:			
		*out &= ~bit; //set low
		break;
	}
}
void _VPMove::set(int _left,int _right,bool noenc){

#if defined _MOVESERVO
  if(_left>0)
	pulses[0].high=(_left/2)+zerol;
  else
	pulses[0].high=(_left/2)+zerol;
  if(_right>0)
	pulses[1].high=(-_right/2)+zeror;
  else
    pulses[1].high=(-_right/2)+zeror;
#elif defined _MOVEHBRIDGE
  uint8_t oldSREG = SREG;
  cli();
  if(_left>=0) {
	_setpin(pinld,HIGH);
	maxL = 255-_left/4;
	startL=255;
	_ldir=1;
    stopL=255;	
	//analogWrite(pinl,_left/4);
	}
  else {
	_setpin(pinld,LOW); 
	//was: OCR0B = 255+_left/4;, but speed was reversed -1000=stop, -1 full
	maxL = 128-_left/8;
	startL=128;
	stopL=0;
	_ldir=-1;
	//analogWrite(pinl,-_left/4);
  }
  if(_right<=0){
	_setpin(pinrd,HIGH);	
	maxR = 255+_right/4;	
	startR=255;
	_rdir=-1;
	stopR=255;
	//analogWrite(pinr2,-_right/4);
	}	
  else{
	_setpin(pinrd,LOW);
	//was OCR0A = 255-_right/4;	but speed was reversed 1=full, 1000=stop
	maxR =128+_right/8;
	startR=128;
	_rdir=1;
	stopR=0;
	//analogWrite(pinr2,-_right/4);
	}
  if(noenc) {
    stopL=maxL;
	stopR=maxR;
	goL=maxL;
	goR=maxR;	
  } 
  else{
	goL=startL;
	goR=startR;    
  } 
  OCR0B=goL;
  OCR0A=goR;    
  SREG = oldSREG;
#endif	
}
	
void _VPMove::setbrake(int _left,int _right){
}
void _VPMove::finish(int _msec){
  delay(_msec);
  set(0,0,true);
}

	

